/*
 *    IIIFReceptionist.java
 *    Copyright (C) 2018 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.greenstone.gsdl3.core;

import org.greenstone.gsdl3.util.*;
import org.greenstone.gsdl3.action.*;
// XML classes
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

// other java classes
import java.io.File;
import java.util.*;

import org.apache.log4j.*;

/** a Receptionist, used for IIIF image server support.
 * This receptionist talks to the message router directly, 
 * instead of via any action, hence no action map is needed.
 * @see the basic Receptionist
 */
public class IIIFReceptionist implements ModuleInterface {
  
  static Logger logger = Logger.getLogger(org.greenstone.gsdl3.core.IIIFReceptionist.class.getName());
  
  /** Instead of a config_params object, only a site_name is needed by iiif receptionist. */
  protected String site_name = null;
    
  /** the configure file of this receptionist passed from the iiif restlet. */
  protected Element iiif_config = null;

  /** the message router that the Receptionist and Actions will talk to */
  protected ModuleInterface mr = null;
  

  public IIIFReceptionist() {

  }
  
  public void cleanUp() {
    if (this.mr != null) {
      
      this.mr.cleanUp();
    }
  }
  
  public void setSiteName(String site_name) {
    this.site_name = site_name;
  }

  /** sets the message router  - it should already be created and
   * configured in the init() of a servlet/restlet (IIIFServerBridge, for example) before being passed to the receptionist*/
  public void setMessageRouter(ModuleInterface mr) {
    this.mr = mr;
  }
  
  /** configures the receptionist */
  public boolean configure(Element config) {
    
    if (this.mr==null) {
      logger.error(" message routers must be set before calling IIIF configure");
      return false;
    }
    if (config == null) {
      logger.error(" IIIF configure file is null");
      return false;
    }
    iiif_config = config;
    
    return true;
  }

    
  protected void resetMessageRouter() {
    // we just need to send a configure request to MR
    Document doc = XMLConverter.newDOM();
    Element mr_request_message = doc.createElement(GSXML.MESSAGE_ELEM);
    Element mr_request = GSXML.createBasicRequest(doc, GSXML.REQUEST_TYPE_SYSTEM, "", null);
    mr_request_message.appendChild(mr_request);
    
    Element system = doc.createElement(GSXML.SYSTEM_ELEM);
    mr_request.appendChild(system);
    system.setAttribute(GSXML.TYPE_ATT, GSXML.SYSTEM_TYPE_CONFIGURE);

    Element response = (Element) this.mr.process(mr_request_message);
    logger.info("*** configure response = "+XMLConverter.getPrettyString(response));
  }

  /** process using strings - just calls process using Elements */
  public String process(String xml_in) {
    
    Node message_node = XMLConverter.getDOM(xml_in);
    Node page = process(message_node);
    return XMLConverter.getString(page);
  }

  //Compose a message/response element used to send back to the IIIFServerBridge servlet. 
  //This method is only used within IIIFReceptionist
  private Element getMessage(Document doc, Element e) {
    Element msg = doc.createElement(GSXML.MESSAGE_ELEM);
    Element response = doc.createElement(GSXML.RESPONSE_ELEM);
    msg.appendChild(response);
    response.appendChild(e);
    return msg;
  }

  /** process - produce xml data in response to a request
   * if something goes wrong, it returns null -
   */
  public Node process(Node message_node) {
    logger.info("*** IIIFReceptionist received request");

    Element message = GSXML.nodeToElement(message_node);
    logger.info("*** " + XMLConverter.getString(message));

    // check that its a correct message tag
    if (!message.getTagName().equals(GSXML.MESSAGE_ELEM)) {
      logger.error(" Invalid message. GSDL message should start with <"+GSXML.MESSAGE_ELEM+">, instead it starts with:"+message.getTagName()+".");
      return IIIFXML.createErrorMessage(IIIFXML.BAD_ARGUMENT, "Internal messaging error");
    }
   
    // get the request out of the message - assume that there is only one
    Element request = (Element)GSXML.getChildByTagName(message, GSXML.REQUEST_ELEM);
    if (request == null) {
      logger.error(" message had no request!");
      return IIIFXML.createErrorMessage(IIIFXML.BAD_ARGUMENT, "Internal messaging error");
    }

    // special case, reset=true for reloading the MR and recept data
    String reset = request.getAttribute("reset");
    if (!reset.equals("")) {
      resetMessageRouter();
      return IIIFXML.createResetResponse(true);
    }

    
    //At this stage, the value of 'to' attribute of the request must be the 'verb'
    //The only thing that the iiif receptionist can be sure is that these verbs are valid, nothing else.
    String verb = request.getAttribute(GSXML.TO_ATT);
    
    if (verb.equals(IIIFXML.GET_RECORD)) {
      return doGetRecord(request);
    }

    // should never get here as verbs were checked in IIIFServerBridge
    return IIIFXML.createErrorMessage(IIIFXML.BAD_VERB, "Unexpected things happened");
    
  }
    
  private Element doGetRecord(Element req){
    logger.info(""); 

    Document doc = XMLConverter.newDOM();
    Element get_record = doc.createElement(IIIFXML.GET_RECORD);

    //HashSet<String> valid_strs = new HashSet<String>(); // ****
    //valid_strs.add(IIIFXML.IDENTIFIER);
    //valid_strs.add(IIIFXML.METADATA_PREFIX);

    NodeList params = GSXML.getChildrenByTagName(req, GSXML.PARAM_ELEM);
    HashMap<String, String> param_map = GSXML.getParamMap(params);    

    // Any need to check all params are valid, like OAI?? // ****
    
    String identifier = param_map.get(IIIFXML.IDENTIFIER);
    
    // get the names
    String[] strs = identifier.split(":", 2);
    if(strs == null || strs.length < 2) {
      logger.error("identifier is not in the form coll:id" + identifier);
      return IIIFXML.createErrorMessage(IIIFXML.ID_DOES_NOT_EXIST, "");
    }    
    String coll_name = strs[0];
    String oid = strs[1];
    
    //re-organize the request element
    // reset the 'to' attribute
    String verb = req.getAttribute(GSXML.TO_ATT);
    req.setAttribute(GSXML.TO_ATT, coll_name + "/" + verb);
    // reset the identifier element
    Element param = GSXML.getNamedElement(req, GSXML.PARAM_ELEM, GSXML.NAME_ATT, IIIFXML.IDENTIFIER);
    if (param != null) {
      param.setAttribute(GSXML.NAME_ATT, IIIFXML.OID);
      param.setAttribute(GSXML.VALUE_ATT, oid);
    }    

    //Now send the request to the message router to process
    Element msg = doc.createElement(GSXML.MESSAGE_ELEM);
    msg.appendChild(doc.importNode(req, true));
    Node result_node = mr.process(msg);
    return GSXML.nodeToElement(result_node);
  }
}

  
